/*
 * vim:ts=8:sw=3:sts=8:noexpandtab:cino=>5n-3f0^-2{2
 */

#include "e_mod_main.h"

/* module private routines */
static Eloquence *_eloquence_new();
static void _eloquence_shutdown(Eloquence *eloquence);
static void _eloquence_config_menu_new(Eloquence *eloquence);

static Eloquence_Face *_eloquence_face_new(E_Container *con);
static void _eloquence_face_free(Eloquence_Face *face);
static void _eloquence_face_enable(Eloquence_Face *face);
static void _eloquence_face_disable(Eloquence_Face *face);
static void  _eloquence_face_menu_new(Eloquence_Face *face);


static void _eloquence_face_cb_gmc_change(void *data, E_Gadman_Client *gmc, E_Gadman_Change  change);

static void _eloquence_face_cb_mouse_down(void *data, Evas *e, Evas_Object *obj, void *event_info);

static void _eloquence_face_cb_menu_edit(void *data, E_Menu *m, E_Menu_Item *mi);

static void _eloquence_face_cb_menu_enabled(void *data, E_Menu *m, E_Menu_Item *mi);
static void _eloquence_face_switch_poll(void *data, float seconds);
static void _eloquence_face_cb_menu_poll_05(void *data, E_Menu *m, E_Menu_Item *mi);
static void _eloquence_face_cb_menu_poll_1(void *data, E_Menu *m, E_Menu_Item *mi);
static void _eloquence_face_cb_menu_poll_2(void *data, E_Menu *m, E_Menu_Item *mi);
static void _eloquence_face_cb_menu_poll_3(void *data, E_Menu *m, E_Menu_Item *mi);
static void _eloquence_face_cb_menu_player_xmms(void *data, E_Menu *m, E_Menu_Item *mi);
static void _eloquence_face_cb_menu_player_bmp(void *data, E_Menu *m, E_Menu_Item *mi);
static void _eloquence_face_cb_menu_player_bmpx(void *data, E_Menu *m, E_Menu_Item *mi);
static void _eloquence_face_cb_menu_player_audacious(void *data, E_Menu *m, E_Menu_Item *mi);
static void _eloquence_face_cb_menu_player_amarok(void *data, E_Menu *m, E_Menu_Item *mi);
static void _eloquence_face_cb_menu_player_mpd(void *data, E_Menu *m, E_Menu_Item *mi);

static void _eloquence_face_cb_pre_menu_theme(void *data, E_Menu *menu);
static void _eloquence_face_cb_post_menu_theme(void *data, E_Menu *menu);
static void _eloquence_face_cb_menu_theme_switch(void *data, E_Menu *menu, E_Menu_Item *item);

static void _theme_menu_load(E_Menu *menu, Eloquence_Face *face);
static int _eloquence_face_switch_theme(void *data, const char *theme);
static void _eloquence_face_switch_player(void *data, unsigned char player);

static void callback_player_play_pause(void *data, Evas_Object *obj, const char *emission, 
					const char *source);
static void callback_player_track_next(void *data, Evas_Object *obj, const char *emission, 
					const char *source);
static void callback_player_track_prev(void *data, Evas_Object *obj, const char *emission, 
					const char *source);
static void callback_player_vol_inc(void *data, Evas_Object *obj, const char *emission, 
					const char *source);
static void callback_player_vol_dec(void *data, Evas_Object *obj, const char *emission, 
					const char *source);
static void callback_player_shuffle(void *data, Evas_Object *obj, const char *emission, 
					const char *source);
static void callback_gui_menu(void *data, Evas_Object *obj, const char *emission, 
					const char *source);



/* Module specific private routines */


static int _eloquence_count;

static E_Config_DD *conf_edd;
static E_Config_DD *conf_face_edd;


/* public module routines. all modules must have these */
E_Module_Api e_modapi =
{
	   E_MODULE_API_VERSION,
	      "Eloquence"
};
	
void *
e_modapi_init(E_Module *module)
{
   Eloquence *eloquence;
  
   /* check module api version */
   if (module->api->version < E_MODULE_API_VERSION)
     {
	e_error_dialog_show(_("Module API Error"),
			    _("Error initializing Module: Eloquence\n"
			      "It requires a minimum module API version of: %i.\n"
			      "The module API advertized by Enlightenment is: %i.\n"
			      "Aborting module."),
			    E_MODULE_API_VERSION,
			    module->api->version);
	return NULL;
     }

   /* actually init eloquence */
   eloquence = _eloquence_new();
   module->config_menu = eloquence->config_menu;
   return eloquence;
}

int
e_modapi_shutdown(E_Module *module)
{
   Eloquence *eloquence;

   if (module->config_menu)
     module->config_menu = NULL;

   eloquence = module->data;
   if (eloquence)
     _eloquence_shutdown(eloquence);

   return 1;
}

int
e_modapi_save(E_Module *module)
{
   Eloquence *eloquence;

   eloquence = module->data;
   e_config_domain_save("module.eloquence", conf_edd, eloquence->conf);
   return 1;
}

int
e_modapi_info(E_Module *module)
{
   char buf[4096];

   snprintf(buf, sizeof(buf), "%s/module_icon.png", e_module_dir_get(module));
   module->icon_file = strdup(buf);
   return 1;
}

int
e_modapi_about(E_Module *module)
{
   e_module_dialog_show(_("Enlightenment Eloquence Module"),
		       _("Shows the title of the song playing in various media players."));
   return 1;
}


/* module private routines */
static Eloquence *
_eloquence_new()
{
   Eloquence *eloquence;
   Evas_List *managers, *l, *l2, *cl;
   E_Menu_Item *mi;

   _eloquence_count = 0;
   eloquence = E_NEW(Eloquence, 1);
   if (!eloquence) return NULL;

   conf_face_edd = E_CONFIG_DD_NEW("Eloquence_Config_Face", Config_Face);
#undef T
#undef D
#define T Config_Face
#define D conf_face_edd
   E_CONFIG_VAL(D, T, enabled, UCHAR);
   E_CONFIG_VAL(D, T, theme, STR);
   E_CONFIG_VAL(D, T, player, CHAR);
   E_CONFIG_VAL(D, T, poll_time, DOUBLE);

   conf_edd = E_CONFIG_DD_NEW("Eloquence_Config", Config);
#undef T
#undef D
#define T Config
#define D conf_edd
   E_CONFIG_LIST(D, T, faces, conf_face_edd);

   eloquence->conf = e_config_domain_load("module.eloquence", conf_edd);
   if (!eloquence->conf)
     {
	eloquence->conf = E_NEW(Config, 1);
     }

   _eloquence_config_menu_new(eloquence);

   managers = e_manager_list();
   cl = eloquence->conf->faces;
   for (l = managers; l; l = l->next)
     {
	E_Manager *man;

	man = l->data;
	for (l2 = man->containers; l2; l2 = l2->next)
	  {
	     E_Container *con;
	     Eloquence_Face *face;

	     con = l2->data;
	     face = _eloquence_face_new(con);
	     if (face)
	       {
		  eloquence->faces = evas_list_append(eloquence->faces, face);
		  if (!cl)
		    {
		       face->conf = E_NEW(Config_Face, 1);
		       face->conf->enabled = 1;
		       face->conf->poll_time = 1.0;
		       face->conf->theme = strdup(THEME_DEFAULT);
		       face->conf->player = PLAYER_XMMS;

		       eloquence->conf->faces = evas_list_append(eloquence->conf->faces, face->conf);
		    }
		  else
		    {
		       face->conf = cl->data;
		       cl = cl->next;
		    }

		  _eloquence_face_menu_new(face);

		  mi = e_menu_item_new(eloquence->config_menu);
		  e_menu_item_label_set(mi, con->name);

		  e_menu_item_submenu_set(mi, face->menu);

		  face->cd = iconv_open ("UTF-8", "ISO-8859-15");
        _eloquence_face_switch_theme(face, face->conf->theme);
        _eloquence_face_switch_player(face, face->conf->player);
       }
	  }
     }
   
   return eloquence;
}

static void
_eloquence_shutdown(Eloquence *eloquence)
{
   Evas_List *list;

   E_CONFIG_DD_FREE(conf_edd);
   E_CONFIG_DD_FREE(conf_face_edd);

   for (list = eloquence->faces; list; list = list->next)
	   _eloquence_face_free(list->data);
   evas_list_free(eloquence->faces);

   e_object_del(E_OBJECT(eloquence->config_menu));

   evas_list_free(eloquence->conf->faces);
   free(eloquence->conf);
   free(eloquence);
}

static void
_eloquence_config_menu_new(Eloquence *eloquence)
{
   eloquence->config_menu = e_menu_new();
}

static Eloquence_Face *
_eloquence_face_new(E_Container *con)
{
   Eloquence_Face *face;
   Evas_Object *o;

   face = E_NEW(Eloquence_Face, 1);
   if (!face) return NULL;

   e_object_ref(E_OBJECT(con));
   face->con = con;

   return face;
}

static void
_eloquence_face_free(Eloquence_Face *face)
{
   if (face->update_timer)
	   ecore_timer_del(face->update_timer);

   e_object_unref(E_OBJECT(face->con));

	if (face->gmc)
      e_object_del(E_OBJECT(face->gmc));
   if (face->eloquence_object)
      evas_object_del(face->eloquence_object);
   if (face->event_object)
      evas_object_del(face->event_object);

   e_object_del(E_OBJECT(face->menu));

   if (face->conf->theme)
      free(face->conf->theme);
   free(face->conf);
   free(face);

   _eloquence_count--;
}

static void
_eloquence_face_enable(Eloquence_Face *face)
{
   face->conf->enabled = 1;

   if (face->eloquence_object)
      evas_object_show(face->eloquence_object);
   if (face->event_object)
      evas_object_show(face->event_object);

   e_config_save_queue();
}

static void
_eloquence_face_disable(Eloquence_Face *face)
{
   face->conf->enabled = 0;

   if (face->eloquence_object)
      evas_object_hide(face->eloquence_object);
   if (face->event_object)
      evas_object_hide(face->event_object);

   e_config_save_queue();
}

static void
_eloquence_face_menu_new(Eloquence_Face *face)
{
   E_Menu *mn, *smn;
   E_Menu_Item *mi;

   mn = e_menu_new();
   face->menu = mn;

   mi = e_menu_item_new(mn);
   e_menu_item_label_set(mi, _("Edit Mode"));
   e_menu_item_callback_set(mi, _eloquence_face_cb_menu_edit, face);

   smn = e_menu_new();
	e_menu_pre_activate_callback_set(smn, _eloquence_face_cb_pre_menu_theme, face);
	e_menu_post_deactivate_callback_set(smn, _eloquence_face_cb_post_menu_theme, face);

   face->theme_menu = smn;
   mi = e_menu_item_new(mn);
   e_menu_item_label_set(mi, _("Theme"));
   e_menu_item_submenu_set(mi, face->theme_menu);

   smn = e_menu_new();
   mi = e_menu_item_new(smn);
   e_menu_item_label_set(mi, _("0.5 seconds"));
   e_menu_item_radio_set(mi, 1);
   e_menu_item_radio_group_set(mi, 1);
   if (face->conf->poll_time == 0.5) e_menu_item_toggle_set(mi, 1);
   e_menu_item_callback_set(mi, _eloquence_face_cb_menu_poll_05, face);
   mi = e_menu_item_new(smn);
   e_menu_item_label_set(mi, _("1 second"));
   e_menu_item_radio_set(mi, 1);
   e_menu_item_radio_group_set(mi, 1);
   if (face->conf->poll_time == 1.0) e_menu_item_toggle_set(mi, 1);
   e_menu_item_callback_set(mi, _eloquence_face_cb_menu_poll_1, face);
   mi = e_menu_item_new(smn);
   e_menu_item_label_set(mi, _("2 seconds"));
   e_menu_item_radio_set(mi, 1);
   e_menu_item_radio_group_set(mi, 1);
   if (face->conf->poll_time == 2.0) e_menu_item_toggle_set(mi, 1);
   e_menu_item_callback_set(mi, _eloquence_face_cb_menu_poll_2, face);
   mi = e_menu_item_new(smn);
   e_menu_item_label_set(mi, _("3 seconds"));
   e_menu_item_radio_set(mi, 1);
   e_menu_item_radio_group_set(mi, 1);
   if (face->conf->poll_time == 3.0) e_menu_item_toggle_set(mi, 1);
   e_menu_item_callback_set(mi, _eloquence_face_cb_menu_poll_3, face);
   face->poll_menu = smn;
   mi = e_menu_item_new(mn);
   e_menu_item_label_set(mi, _("Poll Time"));
   e_menu_item_submenu_set(mi, face->poll_menu);

   smn = e_menu_new();
#ifdef HAVE_XMMS_XMMSCTRL_H
   mi = e_menu_item_new(smn);
   e_menu_item_label_set(mi, _("XMMS"));
   e_menu_item_radio_set(mi, 1);
   e_menu_item_radio_group_set(mi, 1);
   if (face->conf->player == PLAYER_XMMS) e_menu_item_toggle_set(mi, 1);
   e_menu_item_callback_set(mi, _eloquence_face_cb_menu_player_xmms, face);
#endif
#ifdef HAVE_BMP_BEEPCTRL_H
   mi = e_menu_item_new(smn);
   e_menu_item_label_set(mi, _("Beep Media Player"));
   e_menu_item_radio_set(mi, 1);
   e_menu_item_radio_group_set(mi, 1);
   if (face->conf->player == PLAYER_BMP) e_menu_item_toggle_set(mi, 1);
   e_menu_item_callback_set(mi, _eloquence_face_cb_menu_player_bmp, face);
#endif
#ifdef HAVE_AUDACIOUS_BEEPCTRL_H
   mi = e_menu_item_new(smn);
   e_menu_item_label_set(mi, _("Audacious"));
   e_menu_item_radio_set(mi, 1);
   e_menu_item_radio_group_set(mi, 1);
   if (face->conf->player == PLAYER_AUDACIOUS) e_menu_item_toggle_set(mi, 1);
   e_menu_item_callback_set(mi, _eloquence_face_cb_menu_player_audacious, face);
#endif
#ifdef HAVE_BMPX_METADATA_H
   mi = e_menu_item_new(smn);
   e_menu_item_label_set(mi, _("bmpx"));
   e_menu_item_radio_set(mi, 1);
   e_menu_item_radio_group_set(mi, 1);
   if (face->conf->player == PLAYER_BMPX) e_menu_item_toggle_set(mi, 1);
   e_menu_item_callback_set(mi, _eloquence_face_cb_menu_player_bmpx, face);
#endif
#ifdef HAVE_AMAROK
   mi = e_menu_item_new(smn);
   e_menu_item_label_set(mi, _("Amarok (experimental)"));
   e_menu_item_radio_set(mi, 1);
   e_menu_item_radio_group_set(mi, 1);
   if (face->conf->player == PLAYER_AMAROK) e_menu_item_toggle_set(mi, 1);
   e_menu_item_callback_set(mi, _eloquence_face_cb_menu_player_amarok, face);
#endif
#ifdef HAVE_LIBMPD_0_01_LIBMPD_LIBMPD_H
   mi = e_menu_item_new(smn);
   e_menu_item_label_set(mi, _("Music Player Daemon"));
   e_menu_item_radio_set(mi, 1);
   e_menu_item_radio_group_set(mi, 1);
   if (face->conf->player == PLAYER_MPD) e_menu_item_toggle_set(mi, 1);
   e_menu_item_callback_set(mi, _eloquence_face_cb_menu_player_mpd, face);
#endif
   face->player_menu = smn;
   mi = e_menu_item_new(mn);
   e_menu_item_label_set(mi, _("Audio Player Support"));
   e_menu_item_submenu_set(mi, face->player_menu);
}

static void
_eloquence_face_cb_gmc_change(void *data, E_Gadman_Client *gmc, E_Gadman_Change change)
{
   Eloquence_Face *face;
   Evas_Coord x, y, w, h;

   face = data;
   switch (change)
     {
      case E_GADMAN_CHANGE_MOVE_RESIZE:
	 e_gadman_client_geometry_get(face->gmc, &x, &y, &w, &h);
	 evas_object_move(face->eloquence_object, x, y);
	 evas_object_move(face->event_object, x, y);
	 evas_object_resize(face->eloquence_object, w, h);
	 evas_object_resize(face->event_object, w, h);
	 break;
      case E_GADMAN_CHANGE_RAISE:
	 evas_object_raise(face->eloquence_object);
	 evas_object_raise(face->event_object);
	 break;
      case E_GADMAN_CHANGE_EDGE:
      case E_GADMAN_CHANGE_ZONE:
	 /* FIXME
	  * Must we do something here?
	  */
	 break;
     }
}

static void
_eloquence_face_cb_mouse_down(void *data, Evas *e, Evas_Object *obj, void *event_info)
{
/*
   Eloquence_Face *face;
   Evas_Event_Mouse_Down *ev;

   face = data;
   ev = event_info;
   if (ev->button == 3)
     {
	e_menu_activate_mouse(face->menu, e_zone_current_get(face->con),
			      ev->output.x, ev->output.y, 1, 1, NULL,
			      E_MENU_POP_DIRECTION_DOWN);
	e_util_container_fake_mouse_up_all_later(face->con);
     }
*/
}

static void
_eloquence_face_cb_menu_edit(void *data, E_Menu *m, E_Menu_Item *mi)
{
   Eloquence_Face *face;

   face = data;
   e_gadman_mode_set(face->gmc->gadman, E_GADMAN_MODE_EDIT);
}

static void
_eloquence_face_cb_menu_enabled(void *data, E_Menu *m, E_Menu_Item *mi){
   Eloquence_Face *face;
   unsigned char enabled;

   face = data;
   enabled = e_menu_item_toggle_get(mi);
   if ((face->conf->enabled) && (!enabled))
     {
	_eloquence_face_disable(face);
     }
   else if ((!face->conf->enabled) && (enabled))
     {
	_eloquence_face_enable(face);
     }
}

static void
_eloquence_face_switch_poll(void *data, float seconds){
   Eloquence_Face *face = data;
   face->conf->poll_time = seconds;
   e_config_save_queue();
   _eloquence_face_switch_player(face, face->conf->player);
}

static void
_eloquence_face_cb_menu_poll_05(void *data, E_Menu *m, E_Menu_Item *mi){
   _eloquence_face_switch_poll(data, 0.5);
}

static void
_eloquence_face_cb_menu_poll_1(void *data, E_Menu *m, E_Menu_Item *mi){
   _eloquence_face_switch_poll(data, 1.0);
}

static void
_eloquence_face_cb_menu_poll_2(void *data, E_Menu *m, E_Menu_Item *mi){
   _eloquence_face_switch_poll(data, 2.0);
}

static void
_eloquence_face_cb_menu_poll_3(void *data, E_Menu *m, E_Menu_Item *mi){
   _eloquence_face_switch_poll(data, 3.0);
}

static void _eloquence_face_cb_pre_menu_theme(void *data, E_Menu *menu){
  if (menu->realized || !data)
	  return;
  // generate menu
  _theme_menu_load(menu, data);
}

static void _eloquence_face_cb_post_menu_theme(void *data, E_Menu *menu){
  if (menu->parent_item && menu->parent_item->active)
	  return;

  // remove all menuitems
  Evas_List *items;
  for (items = menu->items; items; items = items->next)
	  e_object_del(E_OBJECT(items->data));
  
  evas_list_free(menu->items);
  menu->items = NULL;
}

static void 
_eloquence_face_cb_menu_theme_switch(void *data, E_Menu *menu, E_Menu_Item *item){
  Eloquence_Face *face = data;

  if (strcmp(face->conf->theme, item->label) == 0)
	  return; // theme already active

  if (!_eloquence_face_switch_theme(face, item->label)){
	  // unable to load theme, reset menu to old theme
	  Evas_List *items;
	  for (items = menu->items; items; items = items->next){
		  E_Menu_Item *item = items->data;
		  if (strcmp(face->conf->theme, item->label) == 0){
			  e_menu_item_toggle_set(item, 0);
			  break;
		  }
	  }
  }
  else if (!face->conf->enabled)
	  _eloquence_face_enable(face);
}
	
static void 
_eloquence_face_cb_menu_player_xmms(void *data, E_Menu *m, E_Menu_Item *mi){
   _eloquence_face_switch_player(data, PLAYER_XMMS);
}

static void 
_eloquence_face_cb_menu_player_bmp(void *data, E_Menu *m, E_Menu_Item *mi){
   _eloquence_face_switch_player(data, PLAYER_BMP);
}

static void 
_eloquence_face_cb_menu_player_bmpx(void *data, E_Menu *m, E_Menu_Item *mi){
   _eloquence_face_switch_player(data, PLAYER_BMPX);
}

static void 
_eloquence_face_cb_menu_player_audacious(void *data, E_Menu *m, E_Menu_Item *mi){
   _eloquence_face_switch_player(data, PLAYER_AUDACIOUS);
}

static void 
_eloquence_face_cb_menu_player_amarok(void *data, E_Menu *m, E_Menu_Item *mi){
   _eloquence_face_switch_player(data,PLAYER_AMAROK);
}

static void 
_eloquence_face_cb_menu_player_mpd(void *data, E_Menu *m, E_Menu_Item *mi){
   _eloquence_face_switch_player(data,PLAYER_MPD);
}

void draw_text(void *data){
   Eloquence_Face *face = data;

  size_t insize = (strlen(face->song))+1;
  size_t avail = insize+2;

/* I know this is really ugly...but don't complain:
 * Find better solution or stay quiet! */
   switch (face->conf->player)
     {
#ifdef HAVE_AMAROK
      case PLAYER_AMAROK:
	 face->outbuf[insize-2] = ' ';
         edje_object_part_text_set(face->eloquence_object, "title", face->outbuf);
         return;
         break;	      
#endif
     }

   face->song_utf8 = face->outbuf;
   size_t iconv_errorcode = iconv(face->cd, &face->song, &insize, &face->song_utf8, &avail);
   edje_object_part_text_set(face->eloquence_object, "title", face->outbuf);
}

static void _theme_menu_load(E_Menu *menu, Eloquence_Face *face){
  int theme_found = 0;
  char path[PATH_MAX] = PACKAGE_DATA_DIR"/themes/";
  Ecore_List *files = ecore_file_ls(path);
  if (files){
	  // pointer to the file part
	  char *file_in_path = path + strlen(path);
	  char *file;
	  while ((file = ecore_list_next(files))){
		  if (file[0] == '.')
			  continue; // skip (hide) dot-files
		  strcpy(file_in_path, file);
		  if (ecore_file_is_dir(path))
			  continue; // no sub-directories
		  char *extension = strrchr(file, '.');
		  if (extension && strcasecmp(extension, ".edj") == 0){
			  *extension = '\0'; // strip extension
			  E_Menu_Item *item = e_menu_item_new(menu);
			  e_menu_item_label_set(item, file);
			  e_menu_item_radio_set(item, 1);
			  e_menu_item_radio_group_set(item, 1);
			  if (strcmp(file, face->conf->theme) == 0)
				  e_menu_item_toggle_set(item, 1);
			  e_menu_item_callback_set(item, 
					  _eloquence_face_cb_menu_theme_switch, face);
			  
			  theme_found = 1;
		  }
	  }
	  
	  ecore_list_destroy(files);
  }

  if (!theme_found)
	  printf("eloquence: no themes in %s!\n", PACKAGE_DATA_DIR"/themes");
}

static int
_eloquence_face_switch_theme(void *data, const char *theme){
   Eloquence_Face *face = data;
   Evas_Object *o;
   
   char path[PATH_MAX] = PACKAGE_DATA_DIR"/themes/";
   strcat(strcat(path, theme), ".edj");

   evas_event_freeze(face->con->bg_evas);
   o = edje_object_add(face->con->bg_evas);

   if ( edje_object_file_set(o, path, "eloquence/main") )
   {
	   if (face->event_object)
		   evas_object_del(face->event_object);
	   if (face->eloquence_object)
		   evas_object_del(face->eloquence_object);

	   if (face->gmc)
		   e_object_del(E_OBJECT(face->gmc));
	   
	   // don't free if old and new are the same piece of memory (like on startup)
	   if (face->conf->theme != theme){
		   if (face->conf->theme)
			   free(face->conf->theme);
		   face->conf->theme = strdup(theme);
	   }

	   face->eloquence_object = o;

	   edje_object_signal_callback_add(o, "pause", "pause", callback_player_play_pause, face);
	   edje_object_signal_callback_add(o, "next", "next", callback_player_track_next, face);
	   edje_object_signal_callback_add(o, "prev", "prev", callback_player_track_prev, face);
	   edje_object_signal_callback_add(o, "inc_vol", "inc_vol", callback_player_vol_inc, face);
	   edje_object_signal_callback_add(o, "inc_dec", "inc_dec", callback_player_vol_dec, face);
	   edje_object_signal_callback_add(o, "shuffle", "shuffle", callback_player_shuffle, face);
	   edje_object_signal_callback_add(o, "options", "options", callback_gui_menu, face);

	   if (face->conf->enabled)
		   evas_object_show(o);

	   o = evas_object_rectangle_add(face->con->bg_evas);
	   face->event_object = o;
	   evas_object_layer_set(o, 2);
	   evas_object_repeat_events_set(o, 1);
	   evas_object_color_set(o, 0, 0, 0, 0);

	   if (face->conf->enabled)
		   evas_object_show(o);
	   
	   face->gmc = e_gadman_client_new(face->con->gadman);
	   e_gadman_client_domain_set(face->gmc, "module.eloquence", _eloquence_count++);
	   e_gadman_client_policy_set(face->gmc,
			      E_GADMAN_POLICY_ANYWHERE |
			      E_GADMAN_POLICY_HMOVE |
			      E_GADMAN_POLICY_VMOVE |
			      E_GADMAN_POLICY_HSIZE |
			      E_GADMAN_POLICY_VSIZE);
	   
	   int w, h;
	   edje_object_size_max_get(o, &w, &h);
	   e_gadman_client_max_size_set(face->gmc, w, h);
	   edje_object_size_min_get(o, &w, &h);
	   e_gadman_client_min_size_set(face->gmc, w, h);

	   e_gadman_client_auto_size_set(face->gmc, w, h);
	   e_gadman_client_align_set(face->gmc, 1.0, 1.0);
	   e_gadman_client_resize(face->gmc, 640, 30);
	   e_gadman_client_change_func_set(face->gmc, _eloquence_face_cb_gmc_change, face);
	   e_gadman_client_load(face->gmc);

	   e_config_save_queue();
   }
   else{  // unable to open theme, keep current
	   evas_object_del(o);
	   o = NULL;
   }

   evas_event_thaw(face->con->bg_evas);

   return o != NULL;
}

static void
_eloquence_face_switch_player(void *data, unsigned char player){
   Eloquence_Face *face = data;
   
   if(face->update_timer)
	     ecore_timer_del(face->update_timer);

#ifdef HAVE_LIBMPD_0_01_LIBMPD_LIBMPD_H
   if (face->conf->player == PLAYER_MPD)
	   mpd_disconnect(face);
#endif

   switch (face->conf->player)
     {
#ifdef HAVE_XMMS_XMMSCTRL_H
      case PLAYER_XMMS:
         face->update_timer = ecore_timer_add(face->conf->poll_time, xmms_update_song, face);
         break;	      
#endif
#ifdef HAVE_BMP_BEEPCTRL_H
      case PLAYER_BMP:
         face->update_timer = ecore_timer_add(face->conf->poll_time, bmp_update_song, face);
         break;	      
#endif
#ifdef HAVE_BMPX_METADATA_H
      case PLAYER_BMPX:
         face->update_timer = ecore_timer_add(face->conf->poll_time, bmpx_update_song, face);
         break;	      
#endif
#ifdef HAVE_AUDACIOUS_BEEPCTRL_H
      case PLAYER_AUDACIOUS:
         face->update_timer = ecore_timer_add(face->conf->poll_time, audacious_update_song, face);
         break;	      
#endif
#ifdef HAVE_AMAROK
      case PLAYER_AMAROK:
	 amarok_init(face);
	 face->update_timer = ecore_timer_add(face->conf->poll_time, amarok_update_song, face);
         break;	      
#endif
#ifdef HAVE_LIBMPD_0_01_LIBMPD_LIBMPD_H
      case PLAYER_MPD:
	 mpd_connect(face);
	 face->update_timer = ecore_timer_add(face->conf->poll_time, mpd_update_song, face);
         break;	      
#endif
     }

  face->conf->player = player;
  e_config_save_queue();
}

static void 
callback_player_play_pause(void *data, Evas_Object *obj, const char *emission, const char *source){
   Eloquence_Face *face = data;
   
   switch (face->conf->player)
     {
#ifdef HAVE_XMMS_XMMSCTRL_H
      case PLAYER_XMMS:
         xmms_control_play_pause(face);
         break;	      
#endif
#ifdef HAVE_BMP_BEEPCTRL_H
      case PLAYER_BMP:
         bmp_control_play_pause(face);
         break;	      
#endif
#ifdef HAVE_BMPX_METADATA_H
      case PLAYER_BMPX:
         bmpx_control_play_pause(face);
         break;	      
#endif
#ifdef HAVE_AUDACIOUS_BEEPCTRL_H
      case PLAYER_AUDACIOUS:
         audacious_control_play_pause(face);
         break;	      
#endif
#ifdef HAVE_AMAROK
      case PLAYER_AMAROK:
         amarok_control_play_pause(face);
         break;	      
#endif
#ifdef HAVE_LIBMPD_0_01_LIBMPD_LIBMPD_H
      case PLAYER_MPD:
	 mpd_control_play_pause(face);
         break;              
#endif
     }

}

static void 
callback_player_track_next(void *data, Evas_Object *obj, const char *emission, const char *source){
   Eloquence_Face *face = data;
   
   switch (face->conf->player)
     {
#ifdef HAVE_XMMS_XMMSCTRL_H
      case PLAYER_XMMS:
         xmms_control_track_next(face);
         break;	      
#endif
#ifdef HAVE_BMP_BEEPCTRL_H
      case PLAYER_BMP:
         bmp_control_track_next(face);
         break;	      
#endif
#ifdef HAVE_BMPX_METADATA_H
      case PLAYER_BMPX:
         bmpx_control_track_next(face);
         break;	      
#endif
#ifdef HAVE_AUDACIOUS_BEEPCTRL_H
      case PLAYER_AUDACIOUS:
         audacious_control_track_next(face);
         break;	      
#endif
#ifdef HAVE_AMAROK
      case PLAYER_AMAROK:
         amarok_control_track_next(face);
         break;	      
#endif
#ifdef HAVE_LIBMPD_0_01_LIBMPD_LIBMPD_H
      case PLAYER_MPD:
	 mpd_control_track_next(face);
         break;              
#endif
     }
	
}

static void 
callback_player_track_prev(void *data, Evas_Object *obj, const char *emission, const char *source){
   Eloquence_Face *face = data;
   
   switch (face->conf->player)
     {
#ifdef HAVE_XMMS_XMMSCTRL_H
      case PLAYER_XMMS:
         xmms_control_track_prev(face);
         break;	      
#endif
#ifdef HAVE_BMP_BEEPCTRL_H
      case PLAYER_BMP:
         bmp_control_track_prev(face);
         break;	      
#endif
#ifdef HAVE_BMPX_METADATA_H
      case PLAYER_BMPX:
         bmpx_control_track_prev(face);
         break;	      
#endif
#ifdef HAVE_AUDACIOUS_BEEPCTRL_H
      case PLAYER_AUDACIOUS:
         audacious_control_track_prev(face);
         break;	      
#endif
#ifdef HAVE_AMAROK
      case PLAYER_AMAROK:
         amarok_control_track_prev(face);
         break;	      
#endif
#ifdef HAVE_LIBMPD_0_01_LIBMPD_LIBMPD_H
      case PLAYER_MPD:
	 mpd_control_track_prev(face);
         break;              
#endif
     }
	
}

static void 
callback_player_vol_inc(void *data, Evas_Object *obj, const char *emission, const char *source){
   Eloquence_Face *face = data;
   
   switch (face->conf->player)
     {
#ifdef HAVE_XMMS_XMMSCTRL_H
      case PLAYER_XMMS:
         xmms_control_vol_inc(face);
         break;	      
#endif
#ifdef HAVE_BMP_BEEPCTRL_H
      case PLAYER_BMP:
         bmp_control_vol_inc(face);
         break;	      
#endif
#ifdef HAVE_BMPX_METADATA_H
      case PLAYER_BMPX:
         bmpx_control_vol_inc(face);
         break;	      
#endif
#ifdef HAVE_AUDACIOUS_BEEPCTRL_H
      case PLAYER_AUDACIOUS:
         audacious_control_vol_inc(face);
         break;	      
#endif
#ifdef HAVE_AMAROK
      case PLAYER_AMAROK:
         amarok_control_vol_inc(face);
         break;	      
#endif
#ifdef HAVE_LIBMPD_0_01_LIBMPD_LIBMPD_H
      case PLAYER_MPD:
	 mpd_control_vol_inc(face);
         break;              
#endif
     }

}

static void 
callback_player_vol_dec(void *data, Evas_Object *obj, const char *emission, const char *source){
   Eloquence_Face *face = data;
   
   switch (face->conf->player)
     {
#ifdef HAVE_XMMS_XMMSCTRL_H
      case PLAYER_XMMS:
         xmms_control_vol_dec(face);
         break;	      
#endif
#ifdef HAVE_BMP_BEEPCTRL_H
      case PLAYER_BMP:
         bmp_control_vol_dec(face);
         break;	      
#endif
#ifdef HAVE_BMPX_METADATA_H
      case PLAYER_BMPX:
         bmpx_control_vol_dec(face);
         break;	      
#endif
#ifdef HAVE_AUDACIOUS_BEEPCTRL_H
      case PLAYER_AUDACIOUS:
         audacious_control_vol_dec(face);
         break;	      
#endif
#ifdef HAVE_AMAROK
      case PLAYER_AMAROK:
         amarok_control_vol_dec(face);
         break;	      
#endif
#ifdef HAVE_LIBMPD_0_01_LIBMPD_LIBMPD_H
      case PLAYER_MPD:
	 mpd_control_vol_dec(face);
         break;              
#endif
     }
	
}

static void 
callback_player_shuffle(void *data, Evas_Object *obj, const char *emission, const char *source){
   Eloquence_Face *face = data;
   
   switch (face->conf->player)
     {
#ifdef HAVE_XMMS_XMMSCTRL_H
      case PLAYER_XMMS:
         xmms_control_shuffle(face);
         break;	      
#endif
#ifdef HAVE_BMP_BEEPCTRL_H
      case PLAYER_BMP:
/*  disabled because crashing bmp, bad bug in bmp!!!
	bmp_control_shuffle(face);
*/
         break;	      
#endif
#ifdef HAVE_BMPX_METADATA_H
      case PLAYER_BMPX:
	bmpx_control_shuffle(face);
         break;	      
#endif
#ifdef HAVE_AUDACIOUS_BEEPCTRL_H
      case PLAYER_AUDACIOUS:
/*  disabled because crashing Audacious, bad bug in Audacious!!!
         audacious_control_shuffle(face);
*/
         break;	      
#endif
#ifdef HAVE_AMAROK
      case PLAYER_AMAROK:
         amarok_control_shuffle(face);
         break;	      
#endif
#ifdef HAVE_LIBMPD_0_01_LIBMPD_LIBMPD_H
      case PLAYER_MPD:
	 mpd_control_shuffle(face);
         break;              
#endif
     }

}

static void callback_gui_menu(void *data, Evas_Object *obj, const char *emission, const char *source){
   Eloquence_Face *face = data;
   E_Manager *man;
   E_Container *con;
   int x,y;
   
   man = e_manager_current_get();
   if (!man) return;
   con = e_container_current_get(man);
   if (!con) return;

   e_gadman_client_geometry_get(face->gmc, &x, &y, NULL, NULL);
   
   e_menu_activate_mouse(face->menu, e_zone_current_get(con),
		   x, y, 1, 1, E_MENU_POP_DIRECTION_DOWN, NULL);
   e_util_container_fake_mouse_up_all_later(con);
}

